"""role別conventions YAML制約のE2Eテスト（issue_7063）

実際のYAMLファイル（.claude-nagger/file_conventions.yaml, .claude-nagger/command_conventions.yaml,
.claude-nagger/mcp_conventions.yaml）を読み込み、ImplementationDesignHook.process()経由で
各role × 各ツール制限の動作を検証する。
"""

import json
from pathlib import Path
import pytest
from unittest.mock import patch

from src.domain.hooks.implementation_design_hook import ImplementationDesignHook
from src.domain.services.file_convention_matcher import FileConventionMatcher
from src.domain.services.command_convention_matcher import CommandConventionMatcher
from src.domain.services.mcp_convention_matcher import McpConventionMatcher

# 実際の.claude-nagger/ディレクトリのYAMLを使用
_RULES_DIR = Path(__file__).parent.parent / ".claude-nagger"


def _make_input(tool_name, tool_input=None, transcript_path=None):
    """テスト用input_data生成ヘルパー"""
    data = {
        'tool_name': tool_name,
        'tool_input': tool_input or {},
        'session_id': 'test-session-e2e',
        'tool_use_id': 'toolu_E2E_001',
    }
    if transcript_path:
        data['transcript_path'] = transcript_path
    return data


def _mock_leader(hook):
    """leader判定mock: is_leader_tool_use=True"""
    return patch(
        'src.domain.hooks.implementation_design_hook.is_leader_tool_use',
        return_value=True,
    )


def _mock_role(hook, role_name):
    """特定role判定mock: is_leader=False + _get_caller_roles={role_name}"""
    leader_patch = patch(
        'src.domain.hooks.implementation_design_hook.is_leader_tool_use',
        return_value=False,
    )
    roles_patch = patch.object(
        hook, '_get_caller_roles', return_value={role_name},
    )
    return leader_patch, roles_patch


def _mock_unknown_subagent(hook):
    """role不明subagent: is_leader=False + _get_caller_roles=空"""
    leader_patch = patch(
        'src.domain.hooks.implementation_design_hook.is_leader_tool_use',
        return_value=False,
    )
    roles_patch = patch.object(
        hook, '_get_caller_roles', return_value=set(),
    )
    return leader_patch, roles_patch


def _assert_deny(result):
    """deny判定アサート: decision=block + skip_warn_only=True"""
    assert result['decision'] == 'block', f"Expected block but got {result['decision']}"
    assert result.get('skip_warn_only') is True, "Expected skip_warn_only=True for deny"


def _assert_not_deny(result):
    """deny非該当アサート: approveまたはskip_warn_only非True"""
    if result['decision'] == 'approve':
        return  # approve = OK
    # blockでもskip_warn_only=Trueでなければdenyではない
    assert result.get('skip_warn_only') is not True, \
        f"Unexpected deny: decision={result['decision']}, reason={result.get('reason', '')[:100]}"


class TestDenyByRole:
    """deny確認テスト: 各role制約がprocess()経由で正しくdeny判定される"""

    @pytest.fixture
    def hook(self):
        """実際のYAMLルール（.claude-nagger/）を読み込むhook"""
        h = ImplementationDesignHook()
        h.matcher = FileConventionMatcher(_RULES_DIR / "file_conventions.yaml")
        h.command_matcher = CommandConventionMatcher(_RULES_DIR / "command_conventions.yaml")
        h.mcp_matcher = McpConventionMatcher(_RULES_DIR)
        return h

    # --- ケース1: leader + Edit → deny（leader全ファイル編集禁止） ---
    def test_leader_edit_deny(self, hook, tmp_path):
        """leader + Edit → deny（file_conventions: leader全ファイル編集禁止, scope=leader）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_E2E_001", "name": "Edit"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Edit',
            {'file_path': '/workspace/packages/claude-nagger/src/main.py'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはファイル編集が禁止されています' in result['reason']

    # --- ケース2: leader + Bash("npm install") → deny（leader非gitコマンド禁止） ---
    def test_leader_non_git_command_deny(self, hook, tmp_path):
        """leader + Bash(npm install) → deny（command_conventions: leader非gitコマンド禁止, scope=leader）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_E2E_001", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': 'npm install'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはgitコマンド以外のBash実行が禁止されています' in result['reason']

    # --- ケース3: leader + mcp__redmine_epic_grid__create_epic_tool → deny ---
    def test_leader_redmine_non_allowed_mcp_deny(self, hook, tmp_path):
        """leader + 非許可Redmine MCP → deny（mcp_conventions: leaderRedmine非許可MCP禁止, scope=leader）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001", "name": "mcp__redmine_epic_grid__create_epic_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__create_epic_tool',
            {},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはこのRedmineツールの使用が禁止されています' in result['reason']

    # --- ケース4: tech-lead + Bash("rm -rf /tmp") → deny ---
    def test_tech_lead_destructive_command_deny(self, hook, tmp_path):
        """tech-lead + rm -rf → deny（command_conventions: tech-lead破壊的コマンド禁止, scope=tech-lead）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': 'rm -rf /tmp'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'tech-leadは破壊的コマンドが禁止されています' in result['reason']

    # --- ケース5: researcher + Bash("git push --force origin main") → deny ---
    def test_researcher_destructive_command_deny(self, hook, tmp_path):
        """researcher + git push --force → deny（command_conventions: researcher破壊的コマンド禁止, scope=researcher）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': 'git push --force origin main'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'researcher')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'researcherは破壊的コマンドが禁止されています' in result['reason']

    # --- ケース6: tester + Edit("src/main.py") → deny ---
    def test_tester_production_code_edit_deny(self, hook, tmp_path):
        """tester + Edit(src/) → deny（file_conventions: testerプロダクションコード編集禁止, scope=tester）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Edit"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Edit',
            {'file_path': '/workspace/packages/claude-nagger/src/main.py'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tester')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'testerはプロダクションコード(src/**)の編集が禁止されています' in result['reason']


class TestAllowedOperations:
    """正常操作確認: deny制約に該当しない操作がブロックされないことを検証"""

    @pytest.fixture
    def hook(self):
        """実際のYAMLルール（.claude-nagger/）を読み込むhook"""
        h = ImplementationDesignHook()
        h.matcher = FileConventionMatcher(_RULES_DIR / "file_conventions.yaml")
        h.command_matcher = CommandConventionMatcher(_RULES_DIR / "command_conventions.yaml")
        h.mcp_matcher = McpConventionMatcher(_RULES_DIR)
        return h

    # --- ケース7: leader + Bash("git status") → gitコマンドは非gitルールにマッチしない ---
    def test_leader_git_command_not_denied(self, hook, tmp_path):
        """leader + Bash(git status) → leaderのgitコマンドはdenyされない"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_E2E_001", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': 'git status'},
            str(transcript),
        )
        with _mock_leader(hook):
            # scope=Noneのblock/warnルールは閾値チェックで通過済み扱いにする
            with patch.object(hook, 'is_command_processed', return_value=False):
                result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース8: leader + mcp__redmine_epic_grid__get_issue_detail_tool → 許可リスト内 ---
    def test_leader_allowed_mcp_not_denied(self, hook, tmp_path):
        """leader + 許可リスト内Redmine MCP → denyされない"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__redmine_epic_grid__get_issue_detail_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__get_issue_detail_tool',
            {},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        # 許可リスト内MCPは否定先読みでマッチしないためapprove
        assert result['decision'] == 'approve'

    # --- ケース9: coder + Edit("src/main.py") → coderにはdenyルールなし ---
    def test_coder_edit_not_denied(self, hook, tmp_path):
        """coder + Edit(src/) → coderに対するdenyルールは存在しない"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Edit"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Edit',
            {'file_path': '/workspace/packages/claude-nagger/src/main.py'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'coder')
        with lp, rp:
            # scope=Noneのblock/warnルールはis_rule_processed=Trueでスキップ
            with patch.object(hook, 'is_rule_processed', return_value=True):
                result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース10: tester + Edit("tests/test_main.py") → テストコードは対象外 ---
    def test_tester_test_code_edit_not_denied(self, hook, tmp_path):
        """tester + Edit(tests/) → テストコードはtester制約対象外"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Edit"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Edit',
            {'file_path': '/workspace/packages/claude-nagger/tests/test_main.py'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tester')
        with lp, rp:
            # scope=Noneのblock/warnルールはis_rule_processedでスキップ
            with patch.object(hook, 'is_rule_processed', return_value=True):
                result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース11: tech-lead + Bash("git commit -m 'test'") → 非破壊的コマンド ---
    def test_tech_lead_non_destructive_command_not_denied(self, hook, tmp_path):
        """tech-lead + git commit → 非破壊的gitコマンドはtech-lead制約対象外"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': "git commit -m 'test'"},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            with patch.object(hook, 'is_command_processed', return_value=False):
                result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース12: subagent（role不明）+ Edit → scope=leaderルールは非leaderに適用されない ---
    def test_unknown_subagent_edit_not_denied_by_leader_rule(self, hook, tmp_path):
        """role不明subagent + Edit → scope=leaderルールは適用されない"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Edit"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Edit',
            {'file_path': '/workspace/packages/claude-nagger/src/main.py'},
            str(transcript),
        )
        lp, rp = _mock_unknown_subagent(hook)
        with lp, rp:
            # scope=Noneのblock/warnルールはis_rule_processedでスキップ
            with patch.object(hook, 'is_rule_processed', return_value=True):
                result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース13: leader + 許可リスト内MCPツール追加検証 ---
    def test_leader_allowed_mcp_list_status_tool(self, hook, tmp_path):
        """leader + list_statuses_tool → 許可リスト内"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__redmine_epic_grid__list_statuses_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__list_statuses_tool',
            {},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        assert result['decision'] == 'approve'


class TestDenyByRoleExtended:
    """#7145追加分: role別deny制約のE2Eテスト"""

    @pytest.fixture
    def hook(self):
        """実際のYAMLルール（.claude-nagger/）を読み込むhook"""
        h = ImplementationDesignHook()
        h.matcher = FileConventionMatcher(_RULES_DIR / "file_conventions.yaml")
        h.command_matcher = CommandConventionMatcher(_RULES_DIR / "command_conventions.yaml")
        h.mcp_matcher = McpConventionMatcher(_RULES_DIR)
        return h

    # --- ケース14: pmo + Edit → deny（pmo全ファイル編集禁止） ---
    def test_pmo_edit_deny(self, hook, tmp_path):
        """pmo + Edit → deny（file_conventions: pmo全ファイル編集禁止, scope=pmo）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Edit"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Edit',
            {'file_path': '/workspace/packages/claude-nagger/src/main.py'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'pmo')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'pmoはファイル編集が禁止されています' in result['reason']

    # --- ケース15: tech-lead + Edit → deny（tech-lead全ファイル編集禁止） ---
    def test_tech_lead_edit_deny(self, hook, tmp_path):
        """tech-lead + Edit → deny（file_conventions: tech-lead全ファイル編集禁止, scope=tech-lead）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Edit"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Edit',
            {'file_path': '/workspace/packages/claude-nagger/tests/test_main.py'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'tech-leadはファイル編集が禁止されています' in result['reason']

    # --- ケース16: leader + Serena replace_symbol_body → deny ---
    def test_leader_serena_edit_mcp_deny(self, hook, tmp_path):
        """leader + Serena編集系MCP → deny（mcp_conventions: leaderSerena全般MCP禁止, #7187で拡張）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__serena__replace_symbol_body"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__replace_symbol_body',
            {},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはSerena MCPツールの使用が禁止されています' in result['reason']

    # --- ケース17: pmo + Serena insert_after_symbol → deny ---
    def test_pmo_serena_edit_mcp_deny(self, hook, tmp_path):
        """pmo + Serena編集系MCP → deny（mcp_conventions: pmoSerena編集系MCP禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__serena__insert_after_symbol"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__insert_after_symbol',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'pmo')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'pmoはSerena編集系ツールの使用が禁止されています' in result['reason']

    # --- ケース18: tech-lead + Serena rename_symbol → deny ---
    def test_tech_lead_serena_edit_mcp_deny(self, hook, tmp_path):
        """tech-lead + Serena編集系MCP → deny（mcp_conventions: tech-leadSerena編集系MCP禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__serena__rename_symbol"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__rename_symbol',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'tech-leadはSerena編集系ツールの使用が禁止されています' in result['reason']

    # --- ケース19: tech-lead + Redmine create_epic → deny ---
    def test_tech_lead_redmine_non_allowed_mcp_deny(self, hook, tmp_path):
        """tech-lead + 非許可Redmine MCP → deny（mcp_conventions: tech-leadRedmine非許可MCP禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__redmine_epic_grid__create_epic_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__create_epic_tool',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'tech-leadはこのRedmineツールの使用が禁止されています' in result['reason']

    # --- ケース20: coder + Redmine create_task → deny ---
    def test_coder_redmine_non_allowed_mcp_deny(self, hook, tmp_path):
        """coder + 非許可Redmine MCP → deny（mcp_conventions: coderRedmine非許可MCP禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__redmine_epic_grid__create_task_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__create_task_tool',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'coder')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'coderはこのRedmineツールの使用が禁止されています' in result['reason']

    # --- ケース28: pmo + Bash("npm install") → deny（制約#14 pmo非gitコマンド禁止） ---
    def test_pmo_non_git_command_deny(self, hook, tmp_path):
        """pmo + Bash(npm install) → deny（command_conventions: pmo非gitコマンド禁止, scope=pmo）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': 'npm install'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'pmo')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'pmoはgitコマンド以外のBash実行が禁止されています' in result['reason']

    # --- ケース29: tech-lead + Bash("python3 -m pytest") → deny（制約#15 tech-lead非gitコマンド禁止） ---
    def test_tech_lead_non_git_command_deny(self, hook, tmp_path):
        """tech-lead + Bash(python3 -m pytest) → deny（command_conventions: tech-lead非gitコマンド禁止, scope=tech-lead）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': 'python3 -m pytest'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'tech-leadはgitコマンド以外のBash実行が禁止されています' in result['reason']

    # --- ケース30: tester + Redmine create_task → deny（制約#16 testerRedmine非許可MCP禁止） ---
    def test_tester_redmine_non_allowed_mcp_deny(self, hook, tmp_path):
        """tester + 非許可Redmine MCP → deny（mcp_conventions: testerRedmine非許可MCP禁止, scope=tester）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__redmine_epic_grid__create_task_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__create_task_tool',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tester')
        with lp, rp:
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'testerはこのRedmineツールの使用が禁止されています' in result['reason']


class TestAllowedOperationsExtended:
    """#7145追加分: deny制約に該当しない操作の検証"""

    @pytest.fixture
    def hook(self):
        """実際のYAMLルール（.claude-nagger/）を読み込むhook"""
        h = ImplementationDesignHook()
        h.matcher = FileConventionMatcher(_RULES_DIR / "file_conventions.yaml")
        h.command_matcher = CommandConventionMatcher(_RULES_DIR / "command_conventions.yaml")
        h.mcp_matcher = McpConventionMatcher(_RULES_DIR)
        return h

    # --- ケース21: leader + Serena参照系(find_symbol) → 非deny ---
    def test_leader_serena_read_not_denied(self, hook, tmp_path):
        """leader + Serena参照系MCP → deny（#7187: leaderはSerena全般禁止に拡張）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__serena__find_symbol"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__find_symbol',
            {},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはSerena MCPツールの使用が禁止されています' in result['reason']

    # --- ケース22: tech-lead + Redmine許可リスト内(get_issue_detail) → 非deny ---
    def test_tech_lead_redmine_allowed_mcp_not_denied(self, hook, tmp_path):
        """tech-lead + 許可リスト内Redmine MCP → denyされない"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__redmine_epic_grid__get_issue_detail_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__get_issue_detail_tool',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース23: tech-lead + Redmine許可リスト内(add_issue_comment) → 非deny ---
    def test_tech_lead_redmine_comment_allowed(self, hook, tmp_path):
        """tech-lead + add_issue_comment → 許可リスト内"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__redmine_epic_grid__add_issue_comment_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__add_issue_comment_tool',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース24: coder + Redmine許可リスト内(get_issue_detail) → 非deny ---
    def test_coder_redmine_detail_allowed(self, hook, tmp_path):
        """coder + get_issue_detail → 許可リスト内"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__redmine_epic_grid__get_issue_detail_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__get_issue_detail_tool',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'coder')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース25: coder + Redmine許可リスト内(add_issue_comment) → 非deny ---
    def test_coder_redmine_comment_allowed(self, hook, tmp_path):
        """coder + add_issue_comment → 許可リスト内"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__redmine_epic_grid__add_issue_comment_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__add_issue_comment_tool',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'coder')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース26: coder + Serena編集系 → 非deny（coderにはSerena制約なし） ---
    def test_coder_serena_edit_not_denied(self, hook, tmp_path):
        """coder + Serena編集系MCP → coderにはSerena制約なし"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__serena__replace_symbol_body"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__replace_symbol_body',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'coder')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース27: tech-lead + Serena参照系(get_symbols_overview) → 非deny ---
    def test_tech_lead_serena_read_not_denied(self, hook, tmp_path):
        """tech-lead + Serena参照系MCP → 編集系のみ制約対象"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__serena__get_symbols_overview"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__get_symbols_overview',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース31: pmo + Bash("git status") → 非deny（gitコマンドは許可） ---
    def test_pmo_git_command_not_denied(self, hook, tmp_path):
        """pmo + Bash(git status) → gitコマンドはpmo非gitコマンド禁止ルールにマッチしない"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': 'git status'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'pmo')
        with lp, rp:
            with patch.object(hook, 'is_command_processed', return_value=False):
                result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース32: tech-lead + Bash("git log") → 非deny（gitコマンドは許可） ---
    def test_tech_lead_git_command_not_denied(self, hook, tmp_path):
        """tech-lead + Bash(git log) → gitコマンドはtech-lead非gitコマンド禁止ルールにマッチしない"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [{"type": "tool_use", "id": "toolu_OTHER", "name": "Bash"}]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Bash',
            {'command': 'git log'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tech-lead')
        with lp, rp:
            with patch.object(hook, 'is_command_processed', return_value=False):
                result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース33: tester + Redmine get_issue_detail → 非deny（許可リスト内） ---
    def test_tester_redmine_detail_allowed(self, hook, tmp_path):
        """tester + get_issue_detail → testerRedmine許可リスト内"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "mcp__redmine_epic_grid__get_issue_detail_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__get_issue_detail_tool',
            {},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'tester')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)


class TestLeaderBuiltinToolDeny:
    """leader built-inツール制約テスト（#7187: leader/agent責務分離）"""

    @pytest.fixture
    def hook(self):
        """実際のYAMLルール（.claude-nagger/）を読み込むhook"""
        h = ImplementationDesignHook()
        h.matcher = FileConventionMatcher(_RULES_DIR / "file_conventions.yaml")
        h.command_matcher = CommandConventionMatcher(_RULES_DIR / "command_conventions.yaml")
        h.mcp_matcher = McpConventionMatcher(_RULES_DIR)
        return h

    # --- ケース34: leader + WebFetch → deny ---
    def test_leader_webfetch_deny(self, hook, tmp_path):
        """leader + WebFetch → deny（leaderWebFetchWebSearch禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "WebFetch"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'WebFetch',
            {'url': 'https://example.com', 'prompt': 'test'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはWebFetch/WebSearchの使用が禁止されています' in result['reason']

    # --- ケース35: leader + WebSearch → deny ---
    def test_leader_websearch_deny(self, hook, tmp_path):
        """leader + WebSearch → deny（leaderWebFetchWebSearch禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "WebSearch"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'WebSearch',
            {'query': 'test query'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはWebFetch/WebSearchの使用が禁止されています' in result['reason']

    # --- ケース36: leader + Grep → deny ---
    def test_leader_grep_deny(self, hook, tmp_path):
        """leader + Grep → deny（leaderGrep禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "Grep"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Grep',
            {'pattern': 'test'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはGrepの使用が禁止されています' in result['reason']

    # --- ケース37: leader + Glob → deny ---
    def test_leader_glob_not_denied(self, hook, tmp_path):
        """leader + Glob → 非deny（PMO判断: Globはleader許可ツール）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "Glob"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Glob',
            {'pattern': '**/*.py'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース38: leader + Serena参照系MCP → deny ---
    def test_leader_serena_get_symbols_overview_deny(self, hook, tmp_path):
        """leader + Serena get_symbols_overview → deny（leaderSerena全般MCP禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__serena__get_symbols_overview"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__get_symbols_overview',
            {'relative_path': 'src/main.py'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはSerena MCPツールの使用が禁止されています' in result['reason']

    # --- ケース39: leader + Serena search_for_pattern → deny ---
    def test_leader_serena_search_deny(self, hook, tmp_path):
        """leader + Serena search_for_pattern → deny（leaderSerena全般MCP禁止）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__serena__search_for_pattern"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__search_for_pattern',
            {'substring_pattern': 'test'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_deny(result)
        assert 'leaderはSerena MCPツールの使用が禁止されています' in result['reason']

    # --- ケース40: leader + Serena read_memory → 非deny（許可リスト内） ---
    def test_leader_serena_read_memory_not_denied(self, hook, tmp_path):
        """leader + Serena read_memory → 非deny（leaderSerena許可リスト内）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__serena__read_memory"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__read_memory',
            {'memory_name': 'test'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース41: leader + Serena list_memories → 非deny（許可リスト内） ---
    def test_leader_serena_list_memories_not_denied(self, hook, tmp_path):
        """leader + Serena list_memories → 非deny（leaderSerena許可リスト内）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__serena__list_memories"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__list_memories',
            {},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_not_deny(result)


class TestBuiltinToolAllowForSubagents:
    """built-inツールがsubagentに対してdenyされないことの確認（回帰テスト）"""

    @pytest.fixture
    def hook(self):
        """実際のYAMLルール（.claude-nagger/）を読み込むhook"""
        h = ImplementationDesignHook()
        h.matcher = FileConventionMatcher(_RULES_DIR / "file_conventions.yaml")
        h.command_matcher = CommandConventionMatcher(_RULES_DIR / "command_conventions.yaml")
        h.mcp_matcher = McpConventionMatcher(_RULES_DIR)
        return h

    # --- ケース47: coder + WebSearch → 非deny ---
    def test_coder_websearch_not_denied(self, hook, tmp_path):
        """coder + WebSearch → 非deny（leaderのみ制約対象）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "WebSearch"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'WebSearch',
            {'query': 'test'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'coder')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース48: researcher + Grep → 非deny ---
    def test_researcher_grep_not_denied(self, hook, tmp_path):
        """researcher + Grep → 非deny（leaderのみ制約対象）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_OTHER",
                 "name": "Grep"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Grep',
            {'pattern': 'test'},
            str(transcript),
        )
        lp, rp = _mock_role(hook, 'researcher')
        with lp, rp:
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース49: leader + SendMessage → 非deny（制約対象外） ---
    def test_leader_sendmessage_not_denied(self, hook, tmp_path):
        """leader + SendMessage → 非deny（leaderの許可ツール）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "SendMessage"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'SendMessage',
            {'type': 'message', 'recipient': 'coder', 'content': 'test'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース50: leader + Read → 非deny（leader許可ツール） ---
    def test_leader_task_tools_not_denied(self, hook, tmp_path):
        """leader + TaskCreate → 非deny（leader許可ツール）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "TaskCreate"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'TaskCreate',
            {'subject': 'test task', 'description': 'test'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース51: leader + Redmine参照MCP → 非deny（既存の許可リスト） ---
    def test_leader_redmine_allowed_mcp_still_allowed(self, hook, tmp_path):
        """leader + Redmine get_issue_detail → 非deny（既存回帰テスト）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__redmine_epic_grid__get_issue_detail_tool"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__redmine_epic_grid__get_issue_detail_tool',
            {},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_not_deny(result)

    # --- ケース52: leader + Read → 非deny（leader許可ツール） ---
    # --- ケース52: leader + Read → allow（Readは読み取り専用、file_conventions対象外） ---
    def test_leader_read_not_denied(self, hook, tmp_path):
        """leader + Read → allow（Readは読み取り専用のため、file_conventions対象外）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "Read"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'Read',
            {'file_path': '/tmp/test.py'},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.should_process(input_data)

        # Readは読み取り専用 → should_process=Falseでfile_conventionsスキップ
        assert result is False

    # --- ケース53: leader + Serena initial_instructions → 非deny（許可リスト内） ---
    def test_leader_serena_initial_instructions_not_denied(self, hook, tmp_path):
        """leader + Serena initial_instructions → 非deny（leaderSerena許可リスト内）"""
        transcript = tmp_path / "transcript.jsonl"
        entry = {
            "type": "assistant",
            "message": {"content": [
                {"type": "tool_use", "id": "toolu_E2E_001",
                 "name": "mcp__serena__initial_instructions"}
            ]}
        }
        transcript.write_text(json.dumps(entry) + "\n")

        input_data = _make_input(
            'mcp__serena__initial_instructions',
            {},
            str(transcript),
        )
        with _mock_leader(hook):
            result = hook.process(input_data)

        _assert_not_deny(result)


def test_tech_lead_git_commit_permission_marker():
    """tech-leadのgit commit権限検証マーカー (issue_7217)"""
    # tech-leadがこのファイルの変更をgit commitできることを検証する
    # conventions denyでブロックされないことの確認用
    assert True



#!/usr/bin/env python3
"""match-testコマンドのテスト"""

import pytest
from io import StringIO
from unittest.mock import patch
from application.match_test import MatchTestCommand


class TestMatchTestCommand:
    """MatchTestCommandのテスト"""
    
    def test_file_pattern_match(self):
        """ファイルパターンがマッチする場合"""
        cmd = MatchTestCommand(
            file_path="app/views/test.erb",
            pattern="app/views/**/*.erb"
        )
        
        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
            result = cmd.execute()
        
        assert result == 0
        output = mock_stdout.getvalue()
        assert "Pattern: app/views/**/*.erb" in output
        assert "File: app/views/test.erb" in output
        assert "✅ MATCH" in output
    
    def test_file_pattern_no_match(self):
        """ファイルパターンがマッチしない場合"""
        cmd = MatchTestCommand(
            file_path="app/models/user.rb",
            pattern="app/views/**/*.erb"
        )
        
        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
            result = cmd.execute()
        
        assert result == 0
        output = mock_stdout.getvalue()
        assert "❌ NO MATCH" in output
    
    def test_command_pattern_match(self):
        """コマンドパターンがマッチする場合"""
        cmd = MatchTestCommand(
            command="git push origin main",
            pattern="git push*"
        )
        
        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
            result = cmd.execute()
        
        assert result == 0
        output = mock_stdout.getvalue()
        assert "Pattern: git push*" in output
        assert "Command: git push origin main" in output
        assert "✅ MATCH" in output
    
    def test_command_pattern_no_match(self):
        """コマンドパターンがマッチしない場合"""
        cmd = MatchTestCommand(
            command="npm install",
            pattern="git push*"
        )
        
        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
            result = cmd.execute()
        
        assert result == 0
        output = mock_stdout.getvalue()
        assert "❌ NO MATCH" in output
    
    def test_missing_pattern_error(self):
        """パターン未指定エラー"""
        cmd = MatchTestCommand(
            file_path="test.txt",
            pattern=None
        )
        
        with patch('sys.stderr', new_callable=StringIO) as mock_stderr:
            result = cmd.execute()
        
        assert result == 1
        assert "--pattern is required" in mock_stderr.getvalue()
    
    def test_missing_target_error(self):
        """ターゲット未指定エラー"""
        cmd = MatchTestCommand(
            pattern="*.txt"
        )
        
        with patch('sys.stderr', new_callable=StringIO) as mock_stderr:
            result = cmd.execute()
        
        assert result == 1
        assert "Either --file or --command is required" in mock_stderr.getvalue()
    
    def test_both_targets_error(self):
        """両方指定エラー"""
        cmd = MatchTestCommand(
            file_path="test.txt",
            command="git status",
            pattern="*.txt"
        )
        
        with patch('sys.stderr', new_callable=StringIO) as mock_stderr:
            result = cmd.execute()
        
        assert result == 1
        assert "Cannot specify both --file and --command" in mock_stderr.getvalue()
    
    def test_glob_star_pattern(self):
        """**パターンのテスト"""
        cmd = MatchTestCommand(
            file_path="src/domain/services/file_convention_matcher.py",
            pattern="src/**/*.py"
        )
        
        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
            result = cmd.execute()
        
        assert result == 0
        assert "✅ MATCH" in mock_stdout.getvalue()
    
    def test_single_star_pattern(self):
        """*パターンのテスト"""
        cmd = MatchTestCommand(
            file_path="test.py",
            pattern="*.py"
        )
        
        with patch('sys.stdout', new_callable=StringIO) as mock_stdout:
            result = cmd.execute()
        
        assert result == 0
        assert "✅ MATCH" in mock_stdout.getvalue()



"""subagent override機構のテスト（DB移行後）

#5620: SessionStartup hookのsub-agent別override機構
- SubagentRepository/SessionRepository: DB based state management
- config.yaml overrides: 解決順序（base → subagent_default → subagent_types）、enabled: false
- SessionStartupHook: main agent時は従来通り、subagent時はoverridesメッセージ
- SubagentEventHook: DB登録・削除
- 統合: SubagentStart → PreToolUse（subagent用メッセージ） → SubagentStop
"""

import io
import json
import yaml
import pytest
from pathlib import Path
from unittest.mock import patch, MagicMock

from src.domain.hooks.session_startup_hook import (
    SessionStartupHook,
    _deep_copy_dict,
    _deep_merge,
    _strip_numeric_suffix,
)
from src.domain.hooks.subagent_event_hook import main as subagent_event_main


def _load_real_config():
    """実config.yamlからsession_startup設定を読み込む"""
    config_path = Path(__file__).parent.parent / ".claude-nagger" / "config.yaml"
    with open(config_path, 'r', encoding='utf-8') as f:
        data = yaml.safe_load(f)
    return data.get('session_startup', {})


# ============================================================
# ヘルパー関数テスト
# ============================================================

class TestDeepCopyDict:
    """_deep_copy_dict のテスト"""

    def test_shallow_dict(self):
        """フラットな辞書のコピー"""
        d = {"a": 1, "b": "x"}
        result = _deep_copy_dict(d)
        assert result == d
        assert result is not d

    def test_nested_dict(self):
        """ネストされた辞書のコピー"""
        d = {"a": {"b": {"c": 1}}}
        result = _deep_copy_dict(d)
        assert result == d
        assert result["a"] is not d["a"]
        assert result["a"]["b"] is not d["a"]["b"]

    def test_empty_dict(self):
        """空辞書のコピー"""
        assert _deep_copy_dict({}) == {}

    def test_dict_with_list(self):
        """リスト値を含む辞書の深いコピー"""
        d = {"a": [1, 2, {"b": 3}]}
        result = _deep_copy_dict(d)
        assert result == d
        assert result["a"] is not d["a"]
        assert result["a"][2] is not d["a"][2]

    def test_dict_with_nested_list(self):
        """ネストされたリスト値の深いコピー"""
        d = {"items": [{"name": "x"}, {"name": "y"}]}
        result = _deep_copy_dict(d)
        result["items"][0]["name"] = "modified"
        assert d["items"][0]["name"] == "x"  # 元データに影響しない


class TestDeepMerge:
    """_deep_merge のテスト"""

    def test_simple_override(self):
        """単純な値の上書き"""
        base = {"a": 1, "b": 2}
        override = {"b": 3}
        _deep_merge(base, override)
        assert base == {"a": 1, "b": 3}

    def test_nested_merge(self):
        """ネストされた辞書のマージ"""
        base = {"messages": {"first_time": {"title": "base", "main_text": "base text"}}}
        override = {"messages": {"first_time": {"title": "override"}}}
        _deep_merge(base, override)
        assert base["messages"]["first_time"]["title"] == "override"
        assert base["messages"]["first_time"]["main_text"] == "base text"

    def test_add_new_key(self):
        """新しいキーの追加"""
        base = {"a": 1}
        override = {"b": 2}
        _deep_merge(base, override)
        assert base == {"a": 1, "b": 2}

    def test_override_non_dict_with_dict(self):
        """非辞書値を辞書で上書き"""
        base = {"a": "string"}
        override = {"a": {"nested": True}}
        _deep_merge(base, override)
        assert base["a"] == {"nested": True}

    def test_empty_override(self):
        """空のoverrideはbaseを変更しない"""
        base = {"a": 1}
        _deep_merge(base, {})
        assert base == {"a": 1}


# ============================================================
# SessionStartupHook subagent override テスト（DBベース）
# ============================================================

class TestSessionStartupHookSubagentOverride:
    """SessionStartupHookのsubagent override機構テスト"""

    BASE_CONFIG = _load_real_config()

    def _make_hook(self, config=None):
        """テスト用hookインスタンス生成"""
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def test_resolve_subagent_config_default(self):
        """未定義subagent_typeはsubagent_defaultにフォールバック"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("general-purpose")

        assert resolved["enabled"] is True
        assert resolved["messages"]["first_time"]["title"] == "subagent規約"
        assert "スコープ外のファイルを編集しないこと" in resolved["messages"]["first_time"]["main_text"]
        assert "作業完了後に結果を報告すること" in resolved["messages"]["first_time"]["main_text"]

    def test_resolve_subagent_config_type_specific(self):
        """subagent_typesの設定がsubagent_defaultを上書き"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("Bash")

        assert resolved["enabled"] is True
        assert resolved["messages"]["first_time"]["title"] == "Bash subagent規約"
        assert "破壊的コマンド禁止" in resolved["messages"]["first_time"]["main_text"]

    def test_resolve_subagent_config_disabled(self):
        """enabled: falseの解決"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("Explore")
        assert resolved["enabled"] is False

    def test_resolve_subagent_config_no_overrides(self):
        """overridesセクションなしはbase設定を返す"""
        config = {
            "enabled": True,
            "messages": {"first_time": {"title": "base", "main_text": "base text"}},
        }
        hook = self._make_hook(config)
        resolved = hook._resolve_subagent_config("general-purpose")

        assert resolved["messages"]["first_time"]["title"] == "base"

    def test_resolve_exact_match_takes_priority_over_short_name(self):
        """完全一致が部分一致（短いキー）より優先される"""
        config = {
            "enabled": True,
            "messages": {
                "first_time": {"title": "base", "main_text": "base text"},
            },
            "overrides": {
                "subagent_default": {},
                "subagent_types": {
                    "my-plugin:coder": {
                        "messages": {
                            "first_time": {"title": "完全一致の規約"}
                        }
                    },
                    "coder": {
                        "messages": {
                            "first_time": {"title": "短いキーの規約"}
                        }
                    },
                },
            },
        }
        hook = self._make_hook(config)
        resolved = hook._resolve_subagent_config("my-plugin:coder")

        # 完全一致が優先される
        assert resolved["messages"]["first_time"]["title"] == "完全一致の規約"

    def test_resolve_plain_agent_type_still_works(self):
        """":"を含まない従来のagent_typeが引き続き動作する"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("Bash")

        assert resolved["messages"]["first_time"]["title"] == "Bash subagent規約"
        assert "破壊的コマンド禁止" in resolved["messages"]["first_time"]["main_text"]

    def test_resolve_namespaced_no_match_falls_back_to_default(self):
        """名前空間付きだが短いキーもマッチしない場合はsubagent_defaultにフォールバック"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("my-plugin:unknown-type")

        # subagent_defaultのメッセージが使われる
        assert resolved["messages"]["first_time"]["title"] == "subagent規約"

    def test_resolve_role_takes_priority(self):
        """roleがsubagent_typesにマッチする場合、agent_typeより優先"""
        hook = self._make_hook()
        # role="Bash"でBash用設定を取得、agent_typeは"general-purpose"
        resolved = hook._resolve_subagent_config("general-purpose", role="Bash")

        assert resolved["messages"]["first_time"]["title"] == "Bash subagent規約"
        assert "破壊的コマンド禁止" in resolved["messages"]["first_time"]["main_text"]

    def test_resolve_role_no_match_falls_back_to_agent_type(self):
        """roleがsubagent_typesに無い場合はagent_typeで解決"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("Bash", role="nonexistent-role")

        assert resolved["messages"]["first_time"]["title"] == "Bash subagent規約"

    def test_resolve_role_none_uses_agent_type(self):
        """role=Noneの場合は従来通りagent_typeで解決"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("Bash", role=None)

        assert resolved["messages"]["first_time"]["title"] == "Bash subagent規約"

    def test_resolve_conductor_role(self):
        """conductor roleがsubagent_typesに未定義の場合subagent_defaultにフォールバック"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("general-purpose", role="conductor")

        # conductorはsubagent_typesに未定義 → subagent_default
        assert resolved["messages"]["first_time"]["title"] == "subagent規約"

    def test_resolve_conductor_role_contains_default_rules(self):
        """conductor未定義時はsubagent_defaultの規約が含まれる"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("general-purpose", role="conductor")

        main_text = resolved["messages"]["first_time"]["main_text"]
        assert "作業完了後に結果を報告すること" in main_text
        assert "スコープ外のファイルを編集しないこと" in main_text

    def test_is_session_processed_context_aware_always_false(self):
        """BaseHookのセッションチェックを常にバイパス"""
        hook = self._make_hook()
        result = hook.is_session_processed_context_aware("session-123", {})
        assert result is False


class TestShouldSkipSessionBypass:
    """should_skip_sessionが常にFalseを返し、run()のセッションチェックをバイパスするテスト

    根拠: SessionStartupHookはshould_process()内に独自の重複排除ロジックを持つため、
    BaseHookのセッション処理済みチェックは常にバイパスする必要がある。
    SubagentStartはfire-and-forget(非同期)のため、マーカー依存の条件付きバイパスは
    レースコンディションを引き起こす(issue #5862)。

    リファクタリング(issue #5933): is_session_processed_context_awareオーバーライドから
    should_skip_sessionオーバーライドに移行。run()はshould_skip_session()を呼び出す。
    """

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def test_always_false_without_markers(self):
        """マーカーなし状態で常にFalse"""
        hook = self._make_hook()
        result = hook.should_skip_session("session-123", {})
        assert result is False

    def test_always_false_with_session_marker(self):
        """セッションマーカー存在時でもFalse（BaseHookのチェックをバイパス）"""
        hook = self._make_hook()
        # BaseHookのセッションマーカーを作成
        hook.mark_session_processed("session-with-marker", context_tokens=1000)
        try:
            # SessionStartupHookのshould_skip_sessionは常にFalse
            result = hook.should_skip_session("session-with-marker", {})
            assert result is False
        finally:
            # マーカー削除
            marker_path = hook.get_session_marker_path("session-with-marker")
            if marker_path.exists():
                marker_path.unlink()


class TestSessionStartupHookShouldProcessSubagent:
    """should_processのsubagent検出テスト（DBベース）"""

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def test_subagent_detected_new(self):
        """新規subagent検出時はTrue（DBベース, config定義済みrole）"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-abc"
        mock_record.role = "coder"
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    # agent_idあり = subagent, role=coderはconfig定義済み
                    result = hook.should_process({"session_id": "test-session", "agent_id": "agent-abc"})

        assert result is True
        assert hook._is_subagent is True
        assert hook._current_agent_type == "general-purpose"
        assert hook._current_agent_id == "agent-abc"

    def test_subagent_already_processed(self):
        """処理済みsubagentはFalse（claim_next_unprocessedがNone）"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_subagent_repo.claim_next_unprocessed.return_value = None

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    result = hook.should_process({"session_id": "test-session"})

        assert result is False

    def test_subagent_disabled_type(self):
        """enabled: falseのsubagent種別はFalse"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "Explore"
        mock_record.agent_id = "agent-xyz"
        mock_record.role = None
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    result = hook.should_process({"session_id": "test-session"})

        assert result is False

    def test_no_subagent_main_agent_flow(self):
        """subagentなしの場合はmain agentフロー"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = False
        mock_session_repo.is_processed_context_aware.return_value = False

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    result = hook.should_process({"session_id": "new-session"})

        assert result is True
        assert hook._is_subagent is False

    def test_leader_skips_subagent_blocking(self):
        """issue_7352: agent_id不在（leader）→ subagentブロッキングスキップ"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-abc"
        mock_record.role = None
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    # agent_id不在 = leader → スキップ
                    result = hook.should_process({
                        "session_id": "test-session",
                        "tool_use_id": "toolu_LEADER_001",
                    })

        assert result is False

    def test_subagent_with_agent_id_triggers_blocking(self):
        """issue_7352: agent_idあり（subagent）→ ブロッキング発火"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-abc"
        mock_record.role = "coder"
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    # agent_idあり = subagent → ブロッキング発火
                    result = hook.should_process({
                        "session_id": "test-session",
                        "agent_id": "agent-abc",
                    })

        assert result is True
        assert hook._is_subagent is True
        assert hook._current_agent_id == "agent-abc"

    def test_subagent_config_undefined_skips_with_mark_processed(self):
        """issue_7390: config未定義subagentはスキップ+mark_processed実行"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-abc"
        mock_record.role = None
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    result = hook.should_process({
                        "session_id": "test-session",
                        "agent_id": "agent-abc",
                    })

        assert result is False
        # mark_processedが呼ばれてstartup_processedフラグが正常にマークされること
        mock_subagent_repo.mark_processed.assert_called_once_with("agent-abc")

    def test_subagent_config_defined_role_processes(self):
        """issue_7390: config定義済みroleのsubagentはTrue（正常処理）"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-pmo"
        mock_record.role = "tester"
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    result = hook.should_process({
                        "session_id": "test-session",
                        "agent_id": "agent-pmo",
                    })

        assert result is True
        assert hook._is_subagent is True

    def test_subagent_config_defined_agent_type_processes(self):
        """issue_7390: config定義済みagent_typeのsubagentはTrue"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "Bash"
        mock_record.agent_id = "agent-bash"
        mock_record.role = None
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    result = hook.should_process({
                        "session_id": "test-session",
                        "agent_id": "agent-bash",
                    })

        assert result is True
        assert hook._is_subagent is True

    def test_leader_not_affected_by_config_skip(self):
        """issue_7390: leaderはconfig未定義スキップの影響を受けない"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = False
        mock_session_repo.is_processed_context_aware.return_value = False

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    # agent_id不在 = leader
                    result = hook.should_process({"session_id": "new-session"})

        assert result is True
        assert hook._is_subagent is False


class TestSessionStartupHookProcessSubagent:
    """processのsubagent override テスト（DBベース）"""

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def test_subagent_message_uses_override(self):
        """subagent時はoverride設定のメッセージを使用"""
        hook = self._make_hook()
        hook._is_subagent = True
        hook._current_agent_id = "agent-abc"
        hook._current_agent_type = "general-purpose"
        hook._resolved_config = hook._resolve_subagent_config("general-purpose")
        hook._session_repo = MagicMock()
        hook._subagent_repo = MagicMock()

        result = hook.process({"session_id": "test-session"})

        assert result["decision"] == "block"
        assert "subagent規約" in result["reason"]
        assert "作業完了後に結果を報告すること" in result["reason"]
        # mark_processedが呼ばれたことを確認
        hook._subagent_repo.mark_processed.assert_called_once_with("agent-abc")

    def test_bash_subagent_message(self):
        """Bash subagentは種別固有メッセージを使用"""
        hook = self._make_hook()
        hook._is_subagent = True
        hook._current_agent_id = "agent-bash"
        hook._current_agent_type = "Bash"
        hook._resolved_config = hook._resolve_subagent_config("Bash")
        hook._session_repo = MagicMock()
        hook._subagent_repo = MagicMock()

        result = hook.process({"session_id": "test-session"})

        assert "Bash subagent規約" in result["reason"]
        assert "破壊的コマンド禁止" in result["reason"]

    def test_main_agent_message_unchanged(self):
        """main agent時は従来のメッセージ"""
        hook = self._make_hook()
        hook._is_subagent = False
        hook._resolved_config = None
        hook._session_repo = MagicMock()

        with patch.object(hook, '_get_execution_count', return_value=1):
            with patch.object(hook, '_get_current_context_size', return_value=0):
                result = hook.process({"session_id": "test-session"})

        assert result["decision"] == "block"
        assert "ticket-tasuki 協働規約" in result["reason"]

    def test_subagent_no_register_call(self):
        """subagent処理時はSessionRepository.registerが呼ばれない"""
        hook = self._make_hook()
        hook._is_subagent = True
        hook._current_agent_id = "agent-abc"
        hook._current_agent_type = "general-purpose"
        hook._resolved_config = hook._resolve_subagent_config("general-purpose")
        mock_session_repo = MagicMock()
        hook._session_repo = mock_session_repo
        mock_subagent_repo = MagicMock()
        hook._subagent_repo = mock_subagent_repo

        hook.process({"session_id": "test-session"})

        mock_session_repo.register.assert_not_called()
        # mark_processedが呼ばれたことを確認
        mock_subagent_repo.mark_processed.assert_called_once_with("agent-abc")

    def test_main_agent_creates_db_record(self):
        """main agent処理時はSessionRepository.registerが呼ばれる"""
        hook = self._make_hook()
        hook._is_subagent = False
        hook._resolved_config = None
        mock_session_repo = MagicMock()
        hook._session_repo = mock_session_repo

        with patch.object(hook, '_get_execution_count', return_value=1):
            with patch.object(hook, '_get_current_context_size', return_value=5000):
                hook.process({"session_id": "test-session"})

        mock_session_repo.register.assert_called_once_with("test-session", "SessionStartupHook", 5000)


# ============================================================
# SubagentEventHook テスト（DBベース）
# ============================================================

class TestSubagentEventHook:
    """SubagentEventHookのテスト（DBベース）"""

    def _run_main_with_stdin(self, input_data):
        """stdin経由でmain()を実行するヘルパー。SystemExit例外をキャッチ。"""
        stdin_text = json.dumps(input_data) if isinstance(input_data, dict) else input_data
        with patch('sys.stdin', io.StringIO(stdin_text)):
            with pytest.raises(SystemExit) as exc_info:
                subagent_event_main()
        return exc_info.value.code

    def test_start_event_calls_register(self):
        """SubagentStartイベントでSubagentRepository.registerが呼ばれる"""
        input_data = {
            "hook_event_name": "SubagentStart",
            "session_id": "session-123",
            "agent_id": "agent-abc",
            "agent_type": "general-purpose",
        }

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()

        with patch('src.domain.hooks.subagent_event_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.subagent_event_hook.SubagentRepository', return_value=mock_subagent_repo):
                exit_code = self._run_main_with_stdin(input_data)

        assert exit_code == 0
        mock_subagent_repo.register.assert_called_once()

    def test_stop_event_calls_unregister(self):
        """SubagentStopイベントでSubagentRepository.unregisterが呼ばれる"""
        input_data = {
            "hook_event_name": "SubagentStop",
            "session_id": "session-123",
            "agent_id": "agent-abc",
        }

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()

        with patch('src.domain.hooks.subagent_event_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.subagent_event_hook.SubagentRepository', return_value=mock_subagent_repo):
                exit_code = self._run_main_with_stdin(input_data)

        assert exit_code == 0
        mock_subagent_repo.unregister.assert_called_once_with(
            "agent-abc", agent_transcript_path=None
        )


# ============================================================
# ROLE prefix トランスクリプト解析テスト (#5829)
# ============================================================

class TestParseRoleFromTranscript:
    """_parse_role_from_transcript のユニットテスト"""

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def _write_transcript(self, tmp_path, lines):
        """JSONL形式のトランスクリプトファイルを作成"""
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w', encoding='utf-8') as f:
            for entry in lines:
                f.write(json.dumps(entry, ensure_ascii=False) + '\n')
        return str(transcript)

    def test_team_create_name_extraction(self, tmp_path):
        """TeamCreate方式: input.team_name + input.name → role=name"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_01", "name": "Task",
                 "input": {"team_name": "dev-team", "name": "coder",
                           "subagent_type": "general-purpose", "prompt": "Fix bug."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "coder"

    def test_subagent_type_fallback(self, tmp_path):
        """subagent_typeフォールバック: team_nameなし → role=subagent_type"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_02", "name": "Task",
                 "input": {"subagent_type": "ticket-tasuki:coder", "prompt": "Fix bug."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "ticket-tasuki:coder"

    def test_team_name_priority_over_subagent_type(self, tmp_path):
        """team_name/nameがsubagent_typeより優先される"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_03", "name": "Task",
                 "input": {"team_name": "dev-team", "name": "reviewer",
                           "subagent_type": "general-purpose", "prompt": "Review."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "reviewer"

    def test_no_role_returns_none(self, tmp_path):
        """subagent tool_useがない場合はNone"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "user", "message": {"content": "Just a normal message."}},
            {"type": "assistant", "message": {"content": [
                {"type": "text", "text": "Hello!"},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) is None

    def test_no_subagent_type_no_team_name_returns_none(self, tmp_path):
        """subagent_typeもteam_name/nameもない場合はNone"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_04", "name": "Task",
                 "input": {"prompt": "Do something."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) is None

    def test_skips_non_assistant_entries(self, tmp_path):
        """assistant以外のエントリはスキップされ、assistantのtool_useが取得される"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "user", "message": {"content": "Hello."}},
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_05", "name": "Task",
                 "input": {"subagent_type": "planner", "prompt": "Plan the work."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "planner"

    def test_skips_non_subagent_tool_use(self, tmp_path):
        """SUBAGENT_TOOL_NAMES以外のtool_useはスキップされる"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_06", "name": "Bash",
                 "input": {"command": "ls"}},
                {"type": "tool_use", "id": "toolu_07", "name": "Task",
                 "input": {"subagent_type": "coder", "prompt": "Fix."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "coder"

    def test_none_transcript_path(self):
        """transcript_pathがNoneの場合"""
        hook = self._make_hook()
        assert hook._parse_role_from_transcript(None) is None

    def test_nonexistent_file(self):
        """存在しないファイルパスの場合"""
        hook = self._make_hook()
        assert hook._parse_role_from_transcript("/nonexistent/path.jsonl") is None

    def test_malformed_json_lines(self, tmp_path):
        """不正なJSONL行はスキップ"""
        hook = self._make_hook()
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write("not valid json\n")
            f.write(json.dumps({"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_08", "name": "Task",
                 "input": {"subagent_type": "debug", "prompt": "OK"}},
            ]}}) + '\n')
        assert hook._parse_role_from_transcript(str(transcript)) == "debug"

    def test_empty_file(self, tmp_path):
        """空ファイルの場合"""
        hook = self._make_hook()
        transcript = tmp_path / "transcript.jsonl"
        transcript.write_text("")
        assert hook._parse_role_from_transcript(str(transcript)) is None

    def test_last_tool_use_wins(self, tmp_path):
        """複数のsubagent tool_useがある場合、最後のものが採用される"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_09", "name": "Task",
                 "input": {"subagent_type": "first-role", "prompt": "First."}},
            ]}},
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_10", "name": "Task",
                 "input": {"subagent_type": "last-role", "prompt": "Last."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "last-role"

    def test_agent_tool_name(self, tmp_path):
        """name='Agent'のtool_useも認識される"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_11", "name": "Agent",
                 "input": {"subagent_type": "explorer", "prompt": "Explore."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "explorer"


class TestParseRoleWithParentToolUseId:
    """_parse_role_from_transcript parent_tool_use_id対応テスト（issue_7029）"""

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def _write_transcript(self, tmp_path, lines):
        """JSONL形式のトランスクリプトファイルを作成"""
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w', encoding='utf-8') as f:
            for entry in lines:
                f.write(json.dumps(entry, ensure_ascii=False) + '\n')
        return str(transcript)

    def test_parallel_subagent_correct_role_by_id(self, tmp_path):
        """並列subagent: parent_tool_use_idで正確にrole取得"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_CODER", "name": "Task",
                 "input": {"team_name": "dev", "name": "coder", "prompt": "Code."}},
                {"type": "tool_use", "id": "toolu_TESTER", "name": "Task",
                 "input": {"team_name": "dev", "name": "tester", "prompt": "Test."}},
                {"type": "tool_use", "id": "toolu_REVIEWER", "name": "Task",
                 "input": {"team_name": "dev", "name": "reviewer", "prompt": "Review."}},
            ]}},
        ])
        # 各subagentが自分のtool_useを正確に特定
        assert hook._parse_role_from_transcript(path, "toolu_CODER") == "coder"
        assert hook._parse_role_from_transcript(path, "toolu_TESTER") == "tester"
        assert hook._parse_role_from_transcript(path, "toolu_REVIEWER") == "reviewer"

    def test_parallel_subagent_without_id_gets_last(self, tmp_path):
        """並列subagent: parent_tool_use_idなしは最後のtool_use（従来動作）"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_CODER", "name": "Task",
                 "input": {"team_name": "dev", "name": "coder", "prompt": "Code."}},
                {"type": "tool_use", "id": "toolu_TESTER", "name": "Task",
                 "input": {"team_name": "dev", "name": "tester", "prompt": "Test."}},
            ]}},
        ])
        # parent_tool_use_id未指定は最後のtool_use
        assert hook._parse_role_from_transcript(path) == "tester"

    def test_unmatched_id_falls_back_to_last(self, tmp_path):
        """parentToolUseID不一致時のフォールバック動作"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_CODER", "name": "Task",
                 "input": {"team_name": "dev", "name": "coder", "prompt": "Code."}},
                {"type": "tool_use", "id": "toolu_TESTER", "name": "Task",
                 "input": {"team_name": "dev", "name": "tester", "prompt": "Test."}},
            ]}},
        ])
        # 存在しないIDの場合はフォールバック（最後のtool_use）
        assert hook._parse_role_from_transcript(path, "toolu_NONEXISTENT") == "tester"

    def test_single_subagent_with_id(self, tmp_path):
        """単一subagent: parent_tool_use_idマッチで回帰なし"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_ONLY", "name": "Task",
                 "input": {"subagent_type": "coder", "prompt": "Fix."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path, "toolu_ONLY") == "coder"

    def test_single_subagent_without_id_regression(self, tmp_path):
        """単一subagent: parent_tool_use_idなしの回帰テスト"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_ONLY", "name": "Task",
                 "input": {"subagent_type": "coder", "prompt": "Fix."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "coder"

    def test_none_parent_tool_use_id(self, tmp_path):
        """parent_tool_use_id=Noneは従来動作と同一"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_A", "name": "Task",
                 "input": {"team_name": "t", "name": "alpha", "prompt": "A."}},
                {"type": "tool_use", "id": "toolu_B", "name": "Task",
                 "input": {"team_name": "t", "name": "beta", "prompt": "B."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path, None) == "beta"

    def test_parallel_across_multiple_entries(self, tmp_path):
        """複数assistantエントリにまたがる並列subagent"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_A", "name": "Task",
                 "input": {"team_name": "t", "name": "alpha", "prompt": "A."}},
            ]}},
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_B", "name": "Task",
                 "input": {"team_name": "t", "name": "beta", "prompt": "B."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path, "toolu_A") == "alpha"
        assert hook._parse_role_from_transcript(path, "toolu_B") == "beta"
        # フォールバックは最後
        assert hook._parse_role_from_transcript(path) == "beta"


class TestShouldProcessRoleParsing:
    """should_processでのROLE解析統合テスト（DBベース）"""

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def test_role_parsed_and_stored_in_db(self, tmp_path):
        """transcript解析でroleがDBに保存される（retry_matchがNoneの場合のフォールバック）"""
        hook = self._make_hook()

        # トランスクリプトファイル作成（assistant tool_useからrole抽出）
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write(json.dumps({"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_01", "name": "Task",
                 "input": {"subagent_type": "coder", "prompt": "Fix bug."}},
            ]}}) + '\n')

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-role-test"
        mock_record.role = None
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record
        # retry_matchがNoneを返す（フォールバックテスト）
        mock_subagent_repo.retry_match_from_agent_progress.return_value = None

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    # agent_idあり = subagent
                    result = hook.should_process({
                        "session_id": "test-session",
                        "agent_id": "agent-role-test",
                        "transcript_path": str(transcript),
                    })

        assert result is True
        # update_roleがrole=coderで呼ばれたことを確認
        mock_subagent_repo.update_role.assert_called_once_with("agent-role-test", "coder", "transcript_parse")

    def test_existing_role_not_overwritten(self, tmp_path):
        """マーカーに既存roleがある場合はtranscript解析しない"""
        hook = self._make_hook()

        # transcriptにrole情報があっても無視される
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write(json.dumps({"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_01", "name": "Task",
                 "input": {"subagent_type": "reviewer", "prompt": "Review."}},
            ]}}) + '\n')

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-existing-role"
        mock_record.role = "coder"  # 既にroleがある
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    # agent_idあり = subagent
                    result = hook.should_process({
                        "session_id": "test-session",
                        "agent_id": "agent-existing-role",
                        "transcript_path": str(transcript),
                    })

        assert result is True
        # update_roleはroleで呼ばれない（roleが既にある）
        mock_subagent_repo.update_role.assert_not_called()

    def test_no_role_in_transcript_no_update(self, tmp_path):
        """transcriptにROLEがない場合はupdate_role呼ばれない（retry_matchもNone）
        role=Noneかつconfig未定義→スキップ（issue_7390）"""
        hook = self._make_hook()

        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write(json.dumps({"type": "user", "message": {"content": "No role here."}}) + '\n')

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-no-role"
        mock_record.role = None
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # retry_matchもNoneを返す
        mock_subagent_repo.retry_match_from_agent_progress.return_value = None

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    with patch('domain.services.leader_detection.is_leader_tool_use', return_value=False):
                        result = hook.should_process({
                            "session_id": "test-session",
                            "transcript_path": str(transcript),
                        })

        # role=None + agent_type未定義 → config未マッチでスキップ（issue_7390）
        assert result is False
        mock_subagent_repo.update_role.assert_not_called()
        mock_subagent_repo.mark_processed.assert_called_once_with("agent-no-role")

    def test_role_passed_to_resolve_subagent_config(self, tmp_path):
        """解析されたroleが_resolve_subagent_configに渡される（retry_matchがNoneでフォールバック時）"""
        hook = self._make_hook()

        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write(json.dumps({"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_01", "name": "Task",
                 "input": {"subagent_type": "coder", "prompt": "Work."}},
            ]}}) + '\n')

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-resolve-test"
        mock_record.role = None
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record
        # retry_matchがNoneを返す（フォールバックテスト）
        mock_subagent_repo.retry_match_from_agent_progress.return_value = None

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    with patch.object(hook, '_resolve_subagent_config', return_value={"enabled": True, "messages": {}, "behavior": {}}) as mock_resolve:
                        # agent_idあり = subagent
                        hook.should_process({
                            "session_id": "test-session",
                            "agent_id": "agent-resolve-test",
                            "transcript_path": str(transcript),
                        })

                    # roleがresolve_subagent_configに渡される
                    mock_resolve.assert_called_once_with("general-purpose", role="coder")


# ============================================================
# 案D簡易版（ハイブリッドアプローチ）テスト (#5947)
# ============================================================

class TestRetryMatchFromAgentProgress:
    """retry_match_from_agent_progressのユニットテスト"""

    def test_successful_retry_match(self, tmp_path):
        """agent_progressからの再マッチが成功するケース"""
        from src.infrastructure.db.nagger_state_db import NaggerStateDB
        from src.infrastructure.db.subagent_repository import SubagentRepository

        # tmp_path配下のDBファイルを使用
        db_path = tmp_path / "state.db"
        db = NaggerStateDB(db_path)
        repo = SubagentRepository(db)

        session_id = "test-session"
        agent_id = "agent-retry-test"
        tool_use_id = "toolu_01ABC"

        # subagentを登録（role=NULLで）
        repo.register(agent_id, session_id, "general-purpose", role=None)

        # task_spawnを登録（role付き）
        db.conn.execute(
            """
            INSERT INTO task_spawns (session_id, transcript_index, subagent_type, role, prompt_hash, tool_use_id, created_at)
            VALUES (?, ?, ?, ?, ?, ?, datetime('now'))
            """,
            (session_id, 1, "general-purpose", "coder", "hash123", tool_use_id),
        )
        db.conn.commit()

        # agent_progressを含むtranscriptを作成
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write(json.dumps({
                "type": "progress",
                "parentToolUseID": tool_use_id,
                "data": {
                    "type": "agent_progress",
                    "agentId": agent_id,
                }
            }) + '\n')

        # 再マッチ試行
        result = repo.retry_match_from_agent_progress(session_id, agent_id, str(transcript))

        assert result == "coder"

        # subagentのroleが更新されたか確認
        record = repo.get(agent_id)
        assert record.role == "coder"
        assert record.role_source == "retry_match"

        db.close()

    def test_no_agent_progress_returns_none(self, tmp_path):
        """agent_progressがない場合はNoneを返す"""
        from src.infrastructure.db.nagger_state_db import NaggerStateDB
        from src.infrastructure.db.subagent_repository import SubagentRepository

        db_path = tmp_path / "state.db"
        db = NaggerStateDB(db_path)
        repo = SubagentRepository(db)

        session_id = "test-session"
        agent_id = "agent-no-progress"

        repo.register(agent_id, session_id, "general-purpose", role=None)

        # agent_progressのないtranscript
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write(json.dumps({"type": "user", "message": {"content": "Hello"}}) + '\n')

        result = repo.retry_match_from_agent_progress(session_id, agent_id, str(transcript))
        assert result is None

        db.close()

    def test_no_matching_task_spawn_returns_none(self, tmp_path):
        """task_spawnが見つからない場合はNoneを返す"""
        from src.infrastructure.db.nagger_state_db import NaggerStateDB
        from src.infrastructure.db.subagent_repository import SubagentRepository

        db_path = tmp_path / "state.db"
        db = NaggerStateDB(db_path)
        repo = SubagentRepository(db)

        session_id = "test-session"
        agent_id = "agent-no-task-spawn"
        tool_use_id = "toolu_01XYZ"

        repo.register(agent_id, session_id, "general-purpose", role=None)

        # agent_progressはあるがtask_spawnがない
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write(json.dumps({
                "type": "progress",
                "parentToolUseID": tool_use_id,
                "data": {
                    "type": "agent_progress",
                    "agentId": agent_id,
                }
            }) + '\n')

        result = repo.retry_match_from_agent_progress(session_id, agent_id, str(transcript))
        assert result is None

        db.close()

    def test_already_matched_task_spawn_returns_none(self, tmp_path):
        """既に他のagentにマッチ済みのtask_spawnはNoneを返す"""
        from src.infrastructure.db.nagger_state_db import NaggerStateDB
        from src.infrastructure.db.subagent_repository import SubagentRepository

        db_path = tmp_path / "state.db"
        db = NaggerStateDB(db_path)
        repo = SubagentRepository(db)

        session_id = "test-session"
        agent_id = "agent-late"
        other_agent_id = "agent-early"
        tool_use_id = "toolu_01MATCHED"

        repo.register(agent_id, session_id, "general-purpose", role=None)

        # 既にマッチ済みのtask_spawn
        db.conn.execute(
            """
            INSERT INTO task_spawns (session_id, transcript_index, subagent_type, role, prompt_hash, tool_use_id, matched_agent_id, created_at)
            VALUES (?, ?, ?, ?, ?, ?, ?, datetime('now'))
            """,
            (session_id, 1, "general-purpose", "coder", "hash123", tool_use_id, other_agent_id),
        )
        db.conn.commit()

        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            f.write(json.dumps({
                "type": "progress",
                "parentToolUseID": tool_use_id,
                "data": {
                    "type": "agent_progress",
                    "agentId": agent_id,
                }
            }) + '\n')

        result = repo.retry_match_from_agent_progress(session_id, agent_id, str(transcript))
        assert result is None

        db.close()


class TestShouldProcessRetryMatch:
    """should_processでのretry_match統合テスト"""

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def test_retry_match_called_when_role_is_none(self, tmp_path):
        """role=Noneの場合にretry_match_from_agent_progressが呼ばれる"""
        hook = self._make_hook()

        transcript = tmp_path / "transcript.jsonl"
        transcript.write_text("")

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-retry"
        mock_record.role = None
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        mock_subagent_repo.retry_match_from_agent_progress.return_value = "coder"

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    with patch('domain.services.leader_detection.is_leader_tool_use', return_value=False):
                        result = hook.should_process({
                            "session_id": "test-session",
                            "transcript_path": str(transcript),
                        })

        assert result is True
        mock_subagent_repo.retry_match_from_agent_progress.assert_called_once_with(
            "test-session", "agent-retry", str(transcript)
        )
        # retry_matchが成功したのでupdate_roleは呼ばれない
        mock_subagent_repo.update_role.assert_not_called()

    def test_fallback_to_transcript_parse_when_retry_fails(self, tmp_path):
        """retry_matchが失敗した場合はtranscript解析にフォールバック"""
        hook = self._make_hook()

        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w') as f:
            # config定義済みrole（coder）を使用（issue_7390: 未定義roleはスキップ対象）
            f.write(json.dumps({"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_01", "name": "Task",
                 "input": {"subagent_type": "coder", "prompt": "Fix."}},
            ]}}) + '\n')

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-fallback"
        mock_record.role = None
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record
        mock_subagent_repo.retry_match_from_agent_progress.return_value = None

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    # agent_idあり = subagent
                    result = hook.should_process({
                        "session_id": "test-session",
                        "agent_id": "agent-fallback",
                        "transcript_path": str(transcript),
                    })

        assert result is True
        # retry_matchが失敗したのでtranscript解析でupdate_roleが呼ばれる
        mock_subagent_repo.update_role.assert_called_once_with("agent-fallback", "coder", "transcript_parse")

    def test_retry_match_not_called_when_role_exists(self, tmp_path):
        """既にroleがある場合はretry_matchが呼ばれない"""
        hook = self._make_hook()

        transcript = tmp_path / "transcript.jsonl"
        transcript.write_text("")

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-has-role"
        mock_record.role = "coder"  # 既にroleがある
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    with patch('domain.services.leader_detection.is_leader_tool_use', return_value=False):
                        result = hook.should_process({
                            "session_id": "test-session",
                            "transcript_path": str(transcript),
                        })

        assert result is True
        mock_subagent_repo.retry_match_from_agent_progress.assert_not_called()

    def test_retry_match_not_called_without_transcript_path(self):
        """transcript_pathがない場合はretry_matchが呼ばれない
        role=Noneかつconfig未定義→スキップ（issue_7390）"""
        hook = self._make_hook()

        mock_db = MagicMock()
        mock_subagent_repo = MagicMock()
        mock_session_repo = MagicMock()

        mock_subagent_repo.is_any_active.return_value = True
        mock_record = MagicMock()
        mock_record.agent_type = "general-purpose"
        mock_record.agent_id = "agent-no-transcript"
        mock_record.role = None
        mock_record.startup_processed = False
        mock_subagent_repo.claim_next_unprocessed.return_value = mock_record
        # agent_id直接マッチ用（issue_7531）
        mock_subagent_repo.get.return_value = mock_record

        with patch('src.domain.hooks.session_startup_hook.NaggerStateDB', return_value=mock_db):
            with patch('src.domain.hooks.session_startup_hook.SubagentRepository', return_value=mock_subagent_repo):
                with patch('src.domain.hooks.session_startup_hook.SessionRepository', return_value=mock_session_repo):
                    # agent_idあり = subagent, transcript_pathなし
                    result = hook.should_process({
                        "session_id": "test-session",
                        "agent_id": "agent-no-transcript",
                    })

        # role=None + agent_type未定義 → config未マッチでスキップ（issue_7390）
        assert result is False
        mock_subagent_repo.retry_match_from_agent_progress.assert_not_called()
        mock_subagent_repo.mark_processed.assert_called_once_with("agent-no-transcript")


class TestStripNumericSuffix:
    """_strip_numeric_suffix関数の単体テスト"""

    def test_strip_single_digit(self):
        """末尾-1桁数字を除去"""
        assert _strip_numeric_suffix("tester-2") == "tester"

    def test_strip_different_digit(self):
        """末尾-異なる1桁数字を除去"""
        assert _strip_numeric_suffix("scribe-3") == "scribe"

    def test_no_suffix(self):
        """サフィックスなしはそのまま返す"""
        assert _strip_numeric_suffix("coder") == "coder"

    def test_non_numeric_suffix_unchanged(self):
        """末尾が数字でないハイフン付き名前は変化なし"""
        assert _strip_numeric_suffix("tech-lead") == "tech-lead"

    def test_strip_multi_digit(self):
        """末尾-2桁数字を除去"""
        assert _strip_numeric_suffix("agent-10") == "agent"

    def test_empty_string(self):
        """空文字列はそのまま返す"""
        assert _strip_numeric_suffix("") == ""


class TestResolveSubagentConfigSuffixFallback:
    """_resolve_subagent_configのsuffix除去フォールバックテスト"""

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        """テスト用hookインスタンス生成"""
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def test_exact_match_priority(self):
        """完全一致がsuffix除去より優先される"""
        # config.yamlにtesterがあるので、tester-2用の独自設定を追加
        config = _load_real_config()
        config.setdefault("overrides", {}).setdefault("subagent_types", {})
        config["overrides"]["subagent_types"]["tester-2"] = {
            "messages": {"first_time": {"title": "tester-2専用規約"}}
        }
        hook = self._make_hook(config)
        resolved = hook._resolve_subagent_config("general-purpose", role="tester-2")
        # tester-2の完全一致設定が使われる
        assert resolved["messages"]["first_time"]["title"] == "tester-2専用規約"

    def test_suffix_fallback_role(self):
        """suffix除去でベースロール設定にフォールバック"""
        hook = self._make_hook()
        # config.yamlにtesterはあるがtester-2はない → testerにフォールバック
        resolved = hook._resolve_subagent_config("general-purpose", role="tester-2")
        assert resolved["messages"]["first_time"]["title"] == "tester subagent規約"

    def test_existing_role_no_suffix(self):
        """既存のsuffix無しロール名は変化なく動作（回帰テスト）"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("general-purpose", role="coder")
        assert resolved["messages"]["first_time"]["title"] == "coder subagent規約"

    def test_existing_role_tester(self):
        """testerロールの既存動作確認（回帰テスト）"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("general-purpose", role="tester")
        assert resolved["messages"]["first_time"]["title"] == "tester subagent規約"

    def test_agent_type_suffix_fallback(self):
        """agent_typeにsuffix付きの場合もフォールバック"""
        hook = self._make_hook()
        # roleがNoneでagent_type="tester-2" → testerにフォールバック
        resolved = hook._resolve_subagent_config("tester-2", role=None)
        assert resolved["messages"]["first_time"]["title"] == "tester subagent規約"

    def test_agent_type_no_suffix_still_works(self):
        """agent_typeにsuffixなしの通常動作（回帰テスト）"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("tester")
        assert resolved["messages"]["first_time"]["title"] == "tester subagent規約"

    def test_no_match_returns_default(self):
        """どのルールにもマッチしない場合はsubagent_defaultにフォールバック"""
        hook = self._make_hook()
        resolved = hook._resolve_subagent_config("unknown-agent", role="nonexistent-5")
        # subagent_defaultの設定が使われる
        assert resolved["messages"]["first_time"]["title"] == "subagent規約"
        assert "作業完了後に結果を報告すること" in resolved["messages"]["first_time"]["main_text"]


class TestParseRoleFromTranscriptHyphen:
    """_parse_role_from_transcriptのハイフン付きrole名テスト"""

    BASE_CONFIG = TestSessionStartupHookSubagentOverride.BASE_CONFIG

    def _make_hook(self, config=None):
        """テスト用hookインスタンス生成"""
        config = config if config is not None else self.BASE_CONFIG
        with patch.object(SessionStartupHook, '_load_config', return_value=config):
            return SessionStartupHook()

    def _write_transcript(self, tmp_path, lines):
        """JSONL形式のトランスクリプトファイルを作成"""
        transcript = tmp_path / "transcript.jsonl"
        with open(transcript, 'w', encoding='utf-8') as f:
            for entry in lines:
                f.write(json.dumps(entry, ensure_ascii=False) + '\n')
        return str(transcript)

    def test_role_with_numeric_suffix(self, tmp_path):
        """input.nameに数字サフィックス付き（coder-2）がconfig既知roleに正規化されること（issue_7130）"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_01", "name": "Task",
                 "input": {"team_name": "dev-team", "name": "coder-2",
                           "subagent_type": "general-purpose", "prompt": "Fix bug."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "coder"

    def test_role_without_suffix_still_works(self, tmp_path):
        """input.nameにサフィックスなし（coder）が引き続き動作すること（回帰テスト）"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_02", "name": "Task",
                 "input": {"team_name": "dev-team", "name": "coder",
                           "subagent_type": "general-purpose", "prompt": "Fix bug."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "coder"

    def test_role_with_hyphen_non_numeric(self, tmp_path):
        """input.nameにハイフン付き非数字（tech-lead）がそのまま返ること"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_03", "name": "Task",
                 "input": {"team_name": "dev-team", "name": "tech-lead",
                           "subagent_type": "general-purpose", "prompt": "Review."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "tech-lead"

    def test_role_tester_with_suffix(self, tmp_path):
        """input.nameに数字サフィックス付き（tester-3）がconfig既知roleに正規化されること（issue_7130）"""
        hook = self._make_hook()
        path = self._write_transcript(tmp_path, [
            {"type": "assistant", "message": {"content": [
                {"type": "tool_use", "id": "toolu_04", "name": "Task",
                 "input": {"team_name": "dev-team", "name": "tester-3",
                           "subagent_type": "general-purpose", "prompt": "Run tests."}},
            ]}},
        ])
        assert hook._parse_role_from_transcript(path) == "tester"



"""ファイル編集規約マッチングサービス"""

import os
import yaml
from pathlib import Path
from typing import Dict, List, Optional, Any
from dataclasses import dataclass, field
from wcmatch import glob as wc_glob

from shared.structured_logging import get_logger


@dataclass
class ConventionRule:
    """規約ルール"""
    name: str
    patterns: List[str]
    severity: str  # 'block', 'warn', 'deny'
    message: str
    token_threshold: Optional[int] = None
    scope: Optional[str] = None
    exclude_patterns: List[str] = field(default_factory=list)  # 除外パターンリスト


class FileConventionMatcher:
    """ファイルパスと編集規約のマッチングを行うサービス"""

    def __init__(self, rules_file: Optional[Path] = None, debug: bool = False):
        """
        初期化
        
        Args:
            rules_file: ルール定義ファイルのパス
            debug: デバッグモードフラグ
        """
        if rules_file is None:
            # CLAUDE_PROJECT_DIRを優先、フォールバックはcwd
            project_dir = os.environ.get("CLAUDE_PROJECT_DIR")
            base_path = Path(project_dir) if project_dir else Path.cwd()
            rules_file = base_path / ".claude-nagger" / "file_conventions.yaml"
        
        self.rules_file = Path(rules_file)
        self.debug = debug
        # 統一ログディレクトリを使用（structured_logging）
        self.logger = get_logger("FileConventionMatcher")
        self.rules = self._load_rules()

    def _load_rules(self) -> List[ConventionRule]:
        """ルールファイルを読み込む"""
        self.logger.info(f"Loading rules from: {self.rules_file}")
        
        if not self.rules_file.exists():
            self.logger.error(f"Rules file not found: {self.rules_file}")
            return []
        
        try:
            with open(self.rules_file, 'r', encoding='utf-8') as f:
                data = yaml.safe_load(f)
            
            rules = []
            for rule_data in data.get('rules', []):
                rule = ConventionRule(
                    name=rule_data['name'],
                    patterns=rule_data['patterns'],
                    severity=rule_data.get('severity', 'warn'),
                    message=rule_data['message'],
                    token_threshold=rule_data.get('token_threshold'),
                    scope=rule_data.get('scope'),
                    exclude_patterns=rule_data.get('exclude_patterns', [])
                )
                rules.append(rule)
                self.logger.debug(f"Loaded rule: {rule.name} with patterns: {rule.patterns}")
            
            self.logger.info(f"Successfully loaded {len(rules)} rules")
            return rules
        except yaml.YAMLError as e:
            error_msg = f"設定ファイル構文エラー ({self.rules_file}): {e}"
            self.logger.error(error_msg)
            print(error_msg)
            return []
        except KeyError as e:
            error_msg = f"設定ファイルに必須フィールドがありません ({self.rules_file}): {e}"
            self.logger.error(error_msg)
            print(error_msg)
            return []
        except Exception as e:
            error_msg = f"ルールファイルの読み込みエラー ({self.rules_file}): {e}"
            self.logger.error(error_msg)
            print(error_msg)
            return []

    def matches_pattern(self, file_path: str, patterns: List[str], exclude_patterns: List[str] = None) -> bool:
        """
        ファイルパスがパターンにマッチするか確認
        wcmatch.globを使用して**パターンを正しくサポート

        Args:
            file_path: チェック対象のファイルパス
            patterns: パターンリスト
            exclude_patterns: 除外パターンリスト（マッチすればFalse返却）

        Returns:
            マッチする場合True
        """
        path = Path(file_path)
        
        # 絶対パスの場合、CWDからの相対パスに変換を試みる
        if path.is_absolute():
            try:
                path = path.relative_to(Path.cwd())
                self.logger.info(f"🔄 Converted absolute path to relative: {path}")
            except ValueError:
                # CWD配下にない場合はそのまま使う
                self.logger.info(f"⚠️ Path not under CWD, using as-is: {path}")
        
        normalized_path = str(path.as_posix())
        self.logger.info(f"🔍 PATTERN MATCH DEBUG: Checking file path: {normalized_path}")

        for pattern in patterns:
            self.logger.info(f"  🎯 Testing pattern: {pattern}")

            try:
                # wcmatch.globmatchで**パターンを完全サポート
                if wc_glob.globmatch(normalized_path, pattern, flags=wc_glob.GLOBSTAR):
                    self.logger.info(f"  ✅ Pattern matched: {pattern}")
                    # 除外パターンチェック
                    if exclude_patterns:
                        for exc_pattern in exclude_patterns:
                            if wc_glob.globmatch(normalized_path, exc_pattern, flags=wc_glob.GLOBSTAR):
                                self.logger.info(f"  🚫 Excluded by pattern: {exc_pattern}")
                                return False
                    return True

                self.logger.info(f"  ❌ Pattern not matched: {pattern}")

            except Exception as e:
                # 無効なパターンをスキップ
                self.logger.info(f"  ⚠️ Invalid pattern skipped: {pattern} - {e}")
                continue

        self.logger.info(f"🚫 No patterns matched for: {normalized_path}")
        return False

    def check_file(self, file_path: str) -> List[ConventionRule]:
        """
        ファイルパスに該当する全規約を返す
        
        Args:
            file_path: チェック対象のファイルパス
            
        Returns:
            該当する規約ルールのリスト（なければ空リスト）
        """
        self.logger.info(f"📋 CHECK FILE: {file_path}")
        self.logger.info(f"📊 Total rules loaded: {len(self.rules)}")
        
        matched_rules: List[ConventionRule] = []
        for rule in self.rules:
            self.logger.info(f"🔎 Testing rule: {rule.name}")
            if self.matches_pattern(file_path, rule.patterns, rule.exclude_patterns):
                self.logger.info(f"✅ FILE MATCHED RULE: {rule.name} (severity: {rule.severity})")
                matched_rules.append(rule)
        
        if not matched_rules:
            self.logger.info(f"❌ NO RULES MATCHED FOR FILE: {file_path}")
        
        return matched_rules

    def get_confirmation_message(self, file_path: str) -> List[Dict[str, Any]]:
        """
        確認メッセージを生成（全マッチルール分）
        
        Args:
            file_path: チェック対象のファイルパス
            
        Returns:
            確認メッセージ情報のリスト（なければ空リスト）
        """
        rules = self.check_file(file_path)
        if not rules:
            return []
        
        results = []
        for rule in rules:
            # messageに規約ドキュメントへの参照が既に含まれているため、そのまま使用
            formatted_message = f"""⚠️  {rule.message}"""
            
            results.append({
                'rule_name': rule.name,
                'severity': rule.severity,
                'message': formatted_message,
                'token_threshold': rule.token_threshold,
                'scope': rule.scope
            })
        
        return results

    def reload_rules(self):
        """ルールをリロード"""
        self.rules = self._load_rules()

    def list_rules(self) -> List[Dict[str, Any]]:
        """
        全ルールのリストを返す
        
        Returns:
            ルール情報のリスト
        """
        return [
            {
                'name': rule.name,
                'patterns': rule.patterns,
                'message': rule.message,
                'severity': rule.severity
            }
            for rule in self.rules
        ]


"""SuggestRulesCommand 単体テスト"""

import json
import pytest
from pathlib import Path
from unittest.mock import patch, MagicMock

from application.suggest_rules import SuggestRulesCommand
from domain.services.rule_suggester import PatternSuggestion


# === フィクスチャ ===

@pytest.fixture
def tmp_log_dir(tmp_path):
    """テスト用ログディレクトリ"""
    log_dir = tmp_path / "log"
    log_dir.mkdir()
    return log_dir


def _write_hook_input(log_dir: Path, data: dict, session_id: str = "sess1", index: int = 0) -> Path:
    """hook_input JSONファイルを書き出すヘルパー"""
    filepath = log_dir / f"hook_input_{session_id}_{index:04d}.json"
    filepath.write_text(json.dumps(data, ensure_ascii=False), encoding="utf-8")
    return filepath


def _create_file_inputs(log_dir: Path, paths: list[str], session_id: str = "sess1"):
    """ファイル編集のhook_inputを一括作成"""
    for i, path in enumerate(paths):
        _write_hook_input(log_dir, {
            "tool_name": "Edit",
            "tool_input": {"file_path": path},
        }, session_id=session_id, index=i)


def _create_command_inputs(log_dir: Path, commands: list[str], session_id: str = "sess1", start_index: int = 0):
    """コマンド実行のhook_inputを一括作成"""
    for i, cmd in enumerate(commands):
        _write_hook_input(log_dir, {
            "tool_name": "Bash",
            "tool_input": {"command": cmd},
        }, session_id=session_id, index=start_index + i)


# === executeテスト ===

class TestExecute:
    """CLIコマンド実行テスト"""

    def test_0件時のメッセージ(self, tmp_log_dir, capsys):
        """分析対象なしで適切なメッセージ出力"""
        with patch("application.suggest_rules.RuleSuggester") as MockSuggester:
            mock_instance = MockSuggester.return_value
            mock_instance.analyze.return_value = {
                "file_suggestions": [],
                "command_suggestions": [],
                "stats": {"total_inputs": 0},
            }
            cmd = SuggestRulesCommand()
            result = cmd.execute()

        assert result == 0
        captured = capsys.readouterr()
        assert "規約候補は見つかりませんでした" in captured.err
        assert "hook入力JSONがありません" in captured.err

    def test_0件_入力ありmin_count未満(self, capsys):
        """入力はあるがmin-count未満で0件"""
        with patch("application.suggest_rules.RuleSuggester") as MockSuggester:
            mock_instance = MockSuggester.return_value
            mock_instance.analyze.return_value = {
                "file_suggestions": [
                    PatternSuggestion(category="file", pattern="src/**/*.py", count=2, examples=[]),
                ],
                "command_suggestions": [],
                "stats": {"total_inputs": 5},
            }
            cmd = SuggestRulesCommand(min_count=3)
            result = cmd.execute()

        assert result == 0
        captured = capsys.readouterr()
        assert "規約候補は見つかりませんでした" in captured.err
        assert "min-count=3" in captured.err

    def test_ファイル規約のYAML出力(self, capsys):
        """ファイル規約候補のYAML出力"""
        with patch("application.suggest_rules.RuleSuggester") as MockSuggester:
            mock_instance = MockSuggester.return_value
            mock_instance.analyze.return_value = {
                "file_suggestions": [
                    PatternSuggestion(
                        category="file",
                        pattern="src/domain/hooks/**/*.py",
                        count=12,
                        examples=["src/domain/hooks/base_hook.py"],
                    ),
                ],
                "command_suggestions": [],
                "stats": {"total_inputs": 42},
            }
            cmd = SuggestRulesCommand(min_count=3)
            result = cmd.execute()

        assert result == 0
        captured = capsys.readouterr()
        assert "# suggest-rules 出力" in captured.out
        assert "# 分析対象: 42件のhook入力JSON" in captured.out
        assert "src/domain/hooks/**/*.py" in captured.out
        assert "12回の編集を検出" in captured.out
        assert "severity: warn" in captured.out

    def test_コマンド規約のYAML出力(self, capsys):
        """コマンド規約候補のYAML出力"""
        with patch("application.suggest_rules.RuleSuggester") as MockSuggester:
            mock_instance = MockSuggester.return_value
            mock_instance.analyze.return_value = {
                "file_suggestions": [],
                "command_suggestions": [
                    PatternSuggestion(
                        category="command",
                        pattern="npm test",
                        count=5,
                        examples=["npm test -- --watch"],
                    ),
                ],
                "stats": {"total_inputs": 20},
            }
            cmd = SuggestRulesCommand(min_count=3)
            result = cmd.execute()

        assert result == 0
        captured = capsys.readouterr()
        assert "npm test コマンド規約" in captured.out
        assert "5回の実行を検出" in captured.out

    def test_typeフィルタ_file(self, capsys):
        """--type file でファイル規約のみ出力"""
        with patch("application.suggest_rules.RuleSuggester") as MockSuggester:
            mock_instance = MockSuggester.return_value
            mock_instance.analyze.return_value = {
                "file_suggestions": [
                    PatternSuggestion(category="file", pattern="src/**/*.py", count=10, examples=[]),
                ],
                "command_suggestions": [
                    PatternSuggestion(category="command", pattern="npm", count=8, examples=[]),
                ],
                "stats": {"total_inputs": 30},
            }
            cmd = SuggestRulesCommand(min_count=1, rule_type="file")
            result = cmd.execute()

        assert result == 0
        captured = capsys.readouterr()
        assert "src/**/*.py" in captured.out
        assert "npm" not in captured.out

    def test_typeフィルタ_command(self, capsys):
        """--type command でコマンド規約のみ出力"""
        with patch("application.suggest_rules.RuleSuggester") as MockSuggester:
            mock_instance = MockSuggester.return_value
            mock_instance.analyze.return_value = {
                "file_suggestions": [
                    PatternSuggestion(category="file", pattern="src/**/*.py", count=10, examples=[]),
                ],
                "command_suggestions": [
                    PatternSuggestion(category="command", pattern="npm", count=8, examples=[]),
                ],
                "stats": {"total_inputs": 30},
            }
            cmd = SuggestRulesCommand(min_count=1, rule_type="command")
            result = cmd.execute()

        assert result == 0
        captured = capsys.readouterr()
        assert "npm" in captured.out
        assert "src/**/*.py" not in captured.out

    def test_topN制限(self, capsys):
        """--top N で上位N件のみ出力"""
        file_suggestions = [
            PatternSuggestion(category="file", pattern=f"dir{i}/**/*.py", count=100 - i, examples=[])
            for i in range(5)
        ]
        with patch("application.suggest_rules.RuleSuggester") as MockSuggester:
            mock_instance = MockSuggester.return_value
            mock_instance.analyze.return_value = {
                "file_suggestions": file_suggestions,
                "command_suggestions": [],
                "stats": {"total_inputs": 50},
            }
            cmd = SuggestRulesCommand(min_count=1, top=2)
            result = cmd.execute()

        assert result == 0
        captured = capsys.readouterr()
        assert "dir0/**/*.py" in captured.out
        assert "dir1/**/*.py" in captured.out
        assert "dir2/**/*.py" not in captured.out

    def test_sessionオプション渡し(self):
        """--session がRuleSuggesterに正しく渡される"""
        with patch("application.suggest_rules.RuleSuggester") as MockSuggester:
            mock_instance = MockSuggester.return_value
            mock_instance.analyze.return_value = {
                "file_suggestions": [],
                "command_suggestions": [],
                "stats": {"total_inputs": 0},
            }
            cmd = SuggestRulesCommand(session_id="abc123")
            cmd.execute()

        MockSuggester.assert_called_once_with(session_id="abc123")


# === _format_yaml テスト ===

class TestFormatYaml:
    """YAML出力フォーマットテスト"""

    def test_ヘッダにメタ情報(self):
        """ヘッダにコメント行でメタ情報を含む"""
        cmd = SuggestRulesCommand()
        result = cmd._format_yaml([], [], {"total_inputs": 42})
        assert "# suggest-rules 出力" in result
        assert "# 生成日:" in result
        assert "# 分析対象: 42件のhook入力JSON" in result

    def test_ファイル規約の構造(self):
        """ファイル規約の出力構造"""
        cmd = SuggestRulesCommand()
        suggestion = PatternSuggestion(
            category="file",
            pattern="src/**/*.py",
            count=10,
            examples=["src/a.py"],
        )
        result = cmd._format_yaml([suggestion], [], {"total_inputs": 10})
        assert "patterns:" in result
        assert "- src/**/*.py" in result
        assert "severity: warn" in result

    def test_コマンド規約の構造(self):
        """コマンド規約の出力構造"""
        cmd = SuggestRulesCommand()
        suggestion = PatternSuggestion(
            category="command",
            pattern="git push",
            count=5,
            examples=["git push origin main"],
        )
        result = cmd._format_yaml([], [suggestion], {"total_inputs": 5})
        assert "commands:" in result
        assert "- git push" in result


# === _suggestion_to_rule テスト ===

class TestSuggestionToRule:
    """PatternSuggestion→ルール辞書変換テスト"""

    def test_ファイル規約変換(self):
        """ファイル規約のルール辞書"""
        suggestion = PatternSuggestion(
            category="file", pattern="src/**/*.py", count=7, examples=[]
        )
        rule = SuggestRulesCommand._suggestion_to_rule(suggestion)
        assert rule["name"] == "src/**/*.py 編集規約"
        assert rule["patterns"] == ["src/**/*.py"]
        assert rule["severity"] == "warn"
        assert "7回の編集を検出" in rule["message"]

    def test_コマンド規約変換(self):
        """コマンド規約のルール辞書"""
        suggestion = PatternSuggestion(
            category="command", pattern="docker", count=4, examples=[]
        )
        rule = SuggestRulesCommand._suggestion_to_rule(suggestion)
        assert rule["name"] == "docker コマンド規約"
        assert rule["commands"] == ["docker"]
        assert "4回の実行を検出" in rule["message"]
